// DXMouseCapture.cpp: implementation of the CDXMouseCapture class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "DXMouseCapture.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXMouseCapture::CDXMouseCapture()
{
	m_nSuccessfulSteps = 0;
	m_bInitOK = false;
	m_bCapture = false;
}

CDXMouseCapture::~CDXMouseCapture()
{
	if (m_nSuccessfulSteps >= 2) {
		m_pMouse->Release();
	}
	if (m_nSuccessfulSteps >= 1) {
		m_pDI->Release();
	}
}

bool CDXMouseCapture::Init(HINSTANCE hInstance, HWND hWnd)
{
	HRESULT hr;
	if (!hWnd) {
		return false;
	}
	if (m_nSuccessfulSteps < 1) {
		hr = DirectInput8Create(hInstance,
								DIRECTINPUT_VERSION,
								IID_IDirectInput8,
								(void**)&m_pDI,
								NULL); 
		if FAILED(hr) 
		{ 
			return false; 
		}
		m_nSuccessfulSteps = 1;
	}
 
	if (m_nSuccessfulSteps < 2) {
		hr = m_pDI->CreateDevice(GUID_SysMouse, &m_pMouse, NULL);
		if (FAILED(hr)) {
			return false; 
		}
		m_nSuccessfulSteps = 2;
	}

	if (m_nSuccessfulSteps < 3) {
		hr = m_pMouse->SetDataFormat(&c_dfDIMouse);
		if (FAILED(hr)) {
			return false;
		}
		m_nSuccessfulSteps = 3;
	}

	if (m_nSuccessfulSteps < 4) {
		hr = m_pMouse->SetCooperativeLevel(hWnd,
				   DISCL_EXCLUSIVE | DISCL_FOREGROUND); 
		if (FAILED(hr)) {
			return false;
		}
		m_nSuccessfulSteps = 4;
	}
	m_bInitOK = true;
	return true;
}

bool CDXMouseCapture::Start()
{
	HRESULT hr;
	if (!m_bInitOK) {
		return false;
	}
	hr = m_pMouse->Acquire();
	if (FAILED(hr)) {
		return false;
	}
	else {
		m_bCapture = true;
		return true;
	}
}

void CDXMouseCapture::Stop()
{
	if (m_bInitOK) {
		HRESULT hr;
		hr = m_pMouse->Unacquire();
		m_bCapture = false;
	}
}

bool CDXMouseCapture::GetAxis(int &nX, int &nY, int &nZ)
{
	if (m_bCapture) {
		HRESULT hr;
		DIMOUSESTATE state;
		hr = m_pMouse->GetDeviceState(sizeof(state), &state);
		if (SUCCEEDED(hr)) {
			nX = state.lX;
			nY = state.lY;
			nZ = state.lZ;
			return true;
		}
	}
	return false;
}

bool CDXMouseCapture::GetButtons(unsigned char &but1, unsigned char &but2)
{
	if (m_bCapture) {
		HRESULT hr;
		DIMOUSESTATE state;
		hr = m_pMouse->GetDeviceState(sizeof(state), &state);
		if (SUCCEEDED(hr)) {
			but1 = state.rgbButtons[0];
      but2 = state.rgbButtons[1];
			return true;
		}
	}
	return false;
}

bool CDXMouseCapture::IsCapturing() const
{
	return m_bCapture;
}
