// objects.cpp

#include "stdafx.h"

#include "linecollision.h"



#include "pingpong.h"

#include "objects.h"

void Generalobject::updatePosition() {
  pos = nxbody->getPosition();
}

void Generalobject::updateBody() {
  nxbody->setPosition(pos);
}

void Boxobject::checkCollision(Ball* bl) {

  bool bColl;
  float t0=0, t1=1;
  // Init vectors
  float PT[3], P[3], DT[3], D[3];
  //float C[3];
  SetCoords(PT,bl->lastPos.data.x,bl->lastPos.data.y,bl->lastPos.data.z);
  SetCoords(DT,bl->pos.data.x - PT[0],bl->pos.data.y - PT[1],bl->pos.data.z - PT[2]);


  // Compensate Translation
  PT[0] = PT[0] - m_orientationmatrix[12];
  PT[1] = PT[1] - m_orientationmatrix[13];
  PT[2] = PT[2] - m_orientationmatrix[14];

  P[0] = m_orientationmatrix[0]*PT[0] + m_orientationmatrix[1]*PT[1] + m_orientationmatrix[2]*PT[2];
  P[1] = m_orientationmatrix[4]*PT[0] + m_orientationmatrix[5]*PT[1] + m_orientationmatrix[6]*PT[2];
  P[2] = m_orientationmatrix[8]*PT[0] + m_orientationmatrix[9]*PT[1] + m_orientationmatrix[10]*PT[2];

  D[0] = m_orientationmatrix[0]*DT[0] + m_orientationmatrix[1]*DT[1] + m_orientationmatrix[2]*DT[2];
  D[1] = m_orientationmatrix[4]*DT[0] + m_orientationmatrix[5]*DT[1] + m_orientationmatrix[6]*DT[2];
  D[2] = m_orientationmatrix[8]*DT[0] + m_orientationmatrix[9]*DT[1] + m_orientationmatrix[10]*DT[2];

  // normal for contact
  float n1[6], n[3];
  for(int i=0;i<6;i++)
    n1[i] = 1;
  memset(n,0,sizeof(n));

  if (   (P[0]+D[0] > -dimensions[0]) && (P[0]+D[0] < dimensions[0])
      && (P[1]+D[1] > -dimensions[1]) && (P[1]+D[1]-0.015 < dimensions[1])
      && (P[2]+D[2] > -dimensions[2]) && (P[2]+D[2] < dimensions[2]) && (!isRacket)) {
    bColl = true;
    n[1] = 1;
  }
  else {
    // Intersect
    bColl  = ( clip(+D[0],-P[0]-dimensions[0],t0,t1,n1[0]) )
           & ( clip(+D[1],-P[1]-dimensions[1],t0,t1,n1[1]) )
           & ( clip(+D[2],-P[2]-dimensions[2],t0,t1,n1[2]) )
           & ( clip(-D[0],+P[0]-dimensions[0],t0,t1,n1[3]) )
           & ( clip(-D[1],+P[1]-dimensions[1],t0,t1,n1[4]) )
           & ( clip(-D[2],+P[2]-dimensions[2],t0,t1,n1[5]) );

    if (bColl) {
      // calculate normal
      int j = 0;
      float nc = 1;
      for(i=0;i<6;i++) {
        if (n1[i] < nc) {
          nc = n1[i];
          j = i;
        }
      }
      n[j % 3] = 1;
    }
  }

    //n[0] = n2[0] - n1[0];
    //n[1] = n2[1] - n1[1];
    //n[2] = n2[2] - n1[2];

    // Compensate Translation
    PT[0] = PT[0] + m_orientationmatrix[12];
    PT[1] = PT[1] + m_orientationmatrix[13];
    PT[2] = PT[2] + m_orientationmatrix[14];

  if ( bColl ) {
    bl->color[0] = 1.0f;
	bl->color[1] = 0.0f;
	bl->color[2] = 0.0f;
    PlaySound("pong.wav",NULL,SND_FILENAME | SND_ASYNC | SND_NOWAIT);
    
    norm.data.x = m_orientationmatrix[0]*n[0] + m_orientationmatrix[4]*n[1] + m_orientationmatrix[8]*n[2];
    norm.data.y = m_orientationmatrix[1]*n[0] + m_orientationmatrix[5]*n[1] + m_orientationmatrix[9]*n[2];
    norm.data.z = m_orientationmatrix[2]*n[0] + m_orientationmatrix[6]*n[1] + m_orientationmatrix[10]*n[2];
    
    cont.data.x = PT[0] + t0*DT[0];// + n[0]*0.015;
    cont.data.y = PT[1] + t0*DT[1];// + n[1]*0.015;
    cont.data.z = PT[2] + t0*DT[2];// + n[2]*0.015;

    if (isRacket) {
      norm.data.x = m_orientationmatrix[8];
      norm.data.y = m_orientationmatrix[9];
      norm.data.z = m_orientationmatrix[10];
    }

	bl->nxbody->wakeUp(1000);
    NxContact* contact = g_app.scene->createContact(bl->nxbody, NULL, cont, norm);
    contact->setGeometry(0.0f);//-bl->pos.data.y*2);
    contact->setMaterial(0.6f, 0.05f, 0.1f);//bl->dynFriction, bl->staFriction);
	if (isRacket) {
      //contact->applyNormalForce(10,0.5f);
      //contact->setMaterial(g_app.shootpower, 0,0);//bl->dynFriction, bl->staFriction);
		NxrVec3 velo;
		velo.data.x = norm[0];
		velo.data.y = norm[1];
		velo.data.z = norm[2];
		velo = velo * g_app.shootpower;
		bl->nxbody->setVelocity(velo);
    }
	if (isNet) {
		g_app.netcolor[1] = 0.7f;
		g_app.netcolor[2] = 0.7f;
		
		NxrVec3 velo;
		bl->nxbody->getVelocity(velo);
		if(velo.data.z > 0) 
			velo.data.z = -velo.data.z;
		bl->nxbody->setVelocity(velo);
	}
  }
}

Table::Table() {
  isRacket = false;
  isBall = false;
  isTable = true;
  isGround = false;
  isNet = false;
  dynFriction = 0.05f;
  staFriction = 0.05f;
  
  pos.data.x = 0.0f;
  pos.data.y = 0.675f;
  pos.data.z = -0.5f;

  texUpper = new Texture(TEX_TABLEUP);
  texUpper->loadGL();
  
  SetCoords(dimensions,0.75f,0.025f,0.5f);

  nxbody = NULL;
  calculateOrientationMatrix();
}

Table::~Table() {
  delete texUpper;
  //nxshape->release();
}

void Table::render() {
  glPushMatrix();
   calculateOrientationMatrix();
   glMultMatrixf(m_orientationmatrix);
   renderInit();
  glPopMatrix();
}

void Table::calculateOrientationMatrix() {
  glPushMatrix();
   glLoadIdentity();
   glTranslatef(pos.data.x,pos.data.y,pos.data.z);
   glGetFloatv(GL_MODELVIEW_MATRIX,m_orientationmatrix);
  glPopMatrix();
}

void Table::renderInit() {

  glEnable(GL_TEXTURE_2D);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR );	// Linear Filtering
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );	// Linear Filtering
  texUpper->applyGL();
  
  glBegin(GL_TRIANGLE_FAN);

   glNormal3f(0.0f,1.0f,0.0f);
   glColor3f(0.7f,0.7f,0.65f);
   glTexCoord2f(0.,0.); 
   
   glVertex3f(-0.75f,0.025f,-0.5f);

   glTexCoord2f(1.,0.); 
   glVertex3f( 0.75f,0.025f,-0.5f);

   glTexCoord2f(1.,0.9f); 
   glVertex3f( 0.75f,0.025f,0.5f);

   glTexCoord2f(0.,0.9f); 
   glVertex3f(-0.75f,0.025f,0.5f);

  glEnd();

  glDisable(GL_TEXTURE_2D);
  glColor3f(0.4f,0.5f,0.4f);
  //vorderer Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(0.0f,0.0f,-1.0f);
   glVertex3f(-0.75f,-0.025f,-0.5f);
   glVertex3f(-0.75f, 0.025f,-0.5f);
   glVertex3f( 0.75f, 0.025f,-0.5f);
   glVertex3f( 0.75f,-0.025f,-0.5f);
  glEnd();
  //rechter Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(-1.0f,0.0f,0.0f);
   glVertex3f(-0.75f,-0.025f,-0.5f);
   glVertex3f(-0.75f, 0.025f,-0.5f);
   glVertex3f(-0.75f, 0.025f, 0.5f);
   glVertex3f(-0.75f,-0.025f, 0.5f);
  glEnd();
  //linker Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(1.0f,0.0f,-1);
   glVertex3f( 0.75f,-0.025f,-0.5f);
   glVertex3f( 0.75f, 0.025f,-0.5f);
   glVertex3f( 0.75f, 0.025f, 0.5f);
   glVertex3f( 0.75f,-0.025f, 0.5f);
  glEnd();
  glDisable(GL_TEXTURE_2D);
}


Table2::Table2() {
  isRacket = false;
  isBall = false;
  isTable = true;
  isGround = false;
  isNet = false;
  texUpper = new Texture(TEX_TABLEUP);
  texUpper->loadGL();
  pos.data.x = 0.0f;
  pos.data.y = 0.675f;
  pos.data.z = 0.5f;

  SetCoords(dimensions,0.75f,0.025f,0.5f);


  angle = 0;
  calculateOrientationMatrix();

  nxbody = NULL;

}

Table2::~Table2() {
  delete texUpper;
}

void Table2::render() {
  glPushMatrix();
  glMultMatrixf(m_orientationmatrix);
  renderInit();
  glPopMatrix();
  
  //glCallList(listno);
  //glTranslatef(0,0.7f,0.15f);
  // glRotatef(angle,1,0,0);
  // glTranslatef(0,-0.7f,-0.15f);
  
  //glCallList(listno);
  
}


void Table2::renderInit() {


  glEnable(GL_TEXTURE_2D);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR );	// Linear Filtering
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );	// Linear Filtering
  texUpper->applyGL();
  
  glBegin(GL_TRIANGLE_FAN);

   glNormal3f(0.0f,0.0f,-1.0f);
   glColor3f(0.7f,0.7f,0.65f);
   glTexCoord2f(0.,0.); 
   
   glVertex3f(-0.75f,0.025f,0.5f);

   glTexCoord2f(1.,0.); 
   glVertex3f( 0.75f,0.025f,0.5f);

   glTexCoord2f(1.,0.9f); 
   glVertex3f( 0.75f,0.025f,-0.5f);

   glTexCoord2f(0.,0.9f); 
   glVertex3f(-0.75f,0.025f,-0.5f);

  glEnd();

  glDisable(GL_TEXTURE_2D);
  glColor3f(0.4f,0.5f,0.4f);
  //vorderer Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(0.0f,0.0f,-1.0f);
   glVertex3f(-0.75f,-0.025f,-0.5f);
   glVertex3f(-0.75f, 0.025f,-0.5f);
   glVertex3f( 0.75f, 0.025f,-0.5f);
   glVertex3f( 0.75f,-0.025f,-0.5f);
  glEnd();
  //rechter Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(-1.0f,0.0f,0.0f);
   glVertex3f(-0.75f,-0.025f,-0.5f);
   glVertex3f(-0.75f, 0.025f,-0.5f);
   glVertex3f(-0.75f, 0.025f, 0.5f);
   glVertex3f(-0.75f,-0.025f, 0.5f);
  glEnd();
  //linker Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(1.0f,0.0f,-1);
   glVertex3f( 0.75f,-0.025f,-0.5f);
   glVertex3f( 0.75f, 0.025f,-0.5f);
   glVertex3f( 0.75f, 0.025f, 0.5f);
   glVertex3f( 0.75f,-0.025f, 0.5f);
  glEnd();
  //hinterer Rand
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(0.0f,0.0f,-1.0f);
   glVertex3f(-0.75f,-0.025f, 0.5f);
   glVertex3f(-0.75f, 0.025f, 0.5f);
   glVertex3f( 0.75f, 0.025f, 0.5f);
   glVertex3f( 0.75f,-0.025f, 0.5f);
  glEnd();

}

void Table2::calculateOrientationMatrix() {
  glPushMatrix();
   glLoadIdentity();
   glTranslatef(pos.data.x,pos.data.y,pos.data.z);
   glTranslatef(0,0,-0.35f);
   glRotatef(angle,-1,0,0);
   glTranslatef(0,0,0.35f);
   glGetFloatv(GL_MODELVIEW_MATRIX,m_orientationmatrix);
  glPopMatrix();
}

void Table2::setUp() {
  if (angle<90)
    angle += 2;
  else
   angle = 90;
  calculateOrientationMatrix();
}

void Table2::setDown() {
  if (angle>0)
    angle -= 2;
  else
    angle = 0;
  calculateOrientationMatrix();
}


Ball::Ball() {
  isRacket = false;
  isBall = true;
  isTable = false;
  isGround = false;
  isNet = false;
  dynFriction = 0.05f;
  staFriction = 0.05f;
  
  color[0] = 1.0;
  color[1] = 1.0;
  color[2] = 0.8f;

  b = gluNewQuadric();
  r = 0.015f;
  mass = 0.01f;

  pos.data.x = 0.00f;
  pos.data.y = 1.5f;
  pos.data.z = -0.8f;

  lastPos = pos;


  nxbody = g_app.rsdk->createBody();
  nxbody->setMass(mass);
  nxbody->setPosition(pos); 
  nxbody->setUserData((Generalobject*) this);
  g_app.scene->addBody(*nxbody);

  listno = glGenLists(1);
  glNewList(listno, GL_COMPILE);
	renderInit();
	glEndList();
}

Ball::~Ball() {
  nxbody->release();
}

void Ball::updatePosition() {
  lastPos = pos;
  pos = nxbody->getPosition();
}

void Ball::render() {
  glPushMatrix();
  glTranslatef(pos.data.x,pos.data.y,pos.data.z);
  glColor3fv(color);
  glCallList(listno);
  glPopMatrix();
}

void Ball::renderInit() {
  gluSphere(b,r,10,10);
}

void Ball::drawShadow() {
  float height = 0.701f;

  if(pos.data.x < -0.75f || pos.data.x > 0.75f) height = 0.001f;
  else if(pos.data.z < -1.0f || pos.data.z > 1.0f) height = 0.001f;
  else if(pos.data.y < 0.675f) height = 0.001f;

  glPushMatrix();
  glTranslatef(pos.data.x,height,pos.data.z);
  glBegin(GL_TRIANGLE_FAN);
    glColor3f(0.f,0.1f,0.f);
    glNormal3f(0.f,1.f,0.f);

    glVertex3f(-r,0.f,-r);
    glVertex3f( r,0.f,-r);
    glVertex3f( r,0.f, r);
    glVertex3f(-r,0.f, r);
  glEnd();
  glPopMatrix();
}

Ground::Ground() {
  isRacket = false;
  isBall = false;
  isTable = false;
  isGround = true;
  isNet = false;
  dynFriction = 0.6f;
  staFriction = 0.6f;

 
  texGround = new Texture(TEX_GROUND);
  texGround->loadGL();


  

  
  
  nxbody = NULL;
}

Ground::~Ground() {
  delete texGround;
}

void Ground::render() {
  glEnable(GL_TEXTURE_2D);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR );	// Linear Filtering
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );	// Linear Filtering
  texGround->applyGL();
  glBegin(GL_TRIANGLE_FAN);
    glNormal3f(0.f,1.f,0.f);

    glColor3f(0.5f,0.5f,0.5f);
    glTexCoord2f(0.,0.);
    glVertex3f(-20,0.f,-20);

    glTexCoord2f(60.,0.);
    glVertex3f( 20,0.f,-20);

    glTexCoord2f(60.,60.);
    glVertex3f( 20,0.f, 20);

    glTexCoord2f(0.,60.);
    glVertex3f(-20,0.f, 20);

  glEnd();

/*  texGround->applyGL();
  glBegin(GL_TRIANGLE_FAN);
    glColor3f(1,1,1);
    glNormal3f(0.f,1.f,0.f);

    glTexCoord2f(0.,0.);
    glVertex3f(-1.5,0.f,-2.0);

    glColor3f(1,1,1);
    glTexCoord2f(10,0.);
    glVertex3f( 1.5,0.f,-2.0);

    glColor3f(1,1,1);
    glTexCoord2f(10.,10.);
    glVertex3f( 1.5,0.f, 2.0);

    glColor3f(1,1,1);
    glTexCoord2f(0.,10.);
    glVertex3f(-1.5,0.f, 2.0);

  glEnd();


  //glDisable(GL_TEXTURE_2D);
  //glEnable(GL_TEXTURE_2D);
  
  texHorizon->applyGL();
  glBegin(GL_TRIANGLE_FAN);
    glColor3f(1,1,1);
    glNormal3f(0,0,-1);

    glTexCoord2f(0,0);
    glVertex3f( 15,0,10);
    glTexCoord2f(1,0);
    glVertex3f(-15,0,10);
    glTexCoord2f(1,1);
    glVertex3f(-15,10,10);
    glTexCoord2f(0,1);
    glVertex3f(15,10,10);

  glEnd();*/

  
  glDisable(GL_TEXTURE_2D);


}

void Ground::checkCollision(Ball* bl){
  if (bl->pos.data.y <= 0.015) {
	  bl->color[0] = 1.0f;
	  bl->color[1] = 0.0f;
	  bl->color[2] = 0.0f;
    NxrVec3 norm;
    norm.data.x = 0;
    norm.data.y = 1;
    norm.data.z = 0;
    NxrVec3 cont;
    cont.data.x = bl->pos.data.x;
    cont.data.y = 0;
    cont.data.z = bl->pos.data.z;
    NxContact* contact = g_app.scene->createContact(bl->nxbody, nxbody, bl->pos, norm);
    contact->setGeometry(-bl->pos.data.y*2);
    contact->setMaterial(0.3f, 0, 0);
  }
}

Net::Net()
{
  isRacket = false;
  isBall = false;
  isTable = false;
  isGround = false;
  isNet = true;

  nxbody = 0;
  g_app.netcolor[0] = 1.0f;
  g_app.netcolor[1] = 1.0f;
  g_app.netcolor[2] = 1.0f;
  g_app.netcolor[3] = 0.4f;
  pos.data.x = 0.0f;
  pos.data.y = 0.75f;
  pos.data.z = 0.0f;
  SetCoords(dimensions,0.75f,0.05f,0.05f);
  calculateOrientationMatrix();
}

void Net::calculateOrientationMatrix() {
    glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
      glLoadIdentity();
	  glTranslatef(pos.data.x,pos.data.y,pos.data.z);
	  glGetFloatv(GL_MODELVIEW_MATRIX,m_orientationmatrix);
	glPopMatrix();
}

void Net::render()
{
  glPushMatrix();
  glMultMatrixf(m_orientationmatrix);
	
  // Render Mesh (at the end because of alpha)
	glBegin(GL_TRIANGLE_FAN);
		glNormal3f(0,1,0);  // Normal wrong to get brighter white :-)
		glColor4fv(g_app.netcolor);
		
		/*glVertex3f(-0.75f,0.7f,0);
		glVertex3f( 0.75f,0.7f,0);
		glVertex3f( 0.75f,0.8f,0);
		glVertex3f(-0.75f,0.8f,0);*/

		glVertex3f(-dimensions[0],-dimensions[1],0.0f);
		glVertex3f(-dimensions[0],dimensions[1],0.0f);
		glVertex3f( dimensions[0],dimensions[1],0.0f);
		glVertex3f( dimensions[0],-dimensions[1],0.0f);
	glEnd();


	
	glBegin(GL_LINE_STRIP);
		glNormal3f(0,1,0);
		glColor3f(0.1f,0.1f,0.1f);
		
		/*glVertex3f(-0.75f,0.7f,0);
		glVertex3f(-0.75f,0.8f,0);
		glVertex3f( 0.75f,0.8f,0);
		glVertex3f( 0.75f,0.7f,0);*/
		
		glVertex3f(-dimensions[0],-dimensions[1],0);
		glVertex3f(-dimensions[0],dimensions[1],0);
		glVertex3f( dimensions[0],dimensions[1],0);
		glVertex3f( dimensions[0],-dimensions[1],0);
	glEnd();
  glPopMatrix();
}
