// objects.h


// Different objects used within the demo
// Object instances store properties as their position and orientation
// and provide their rendering

#ifndef _OBJECTS_H_
#define _OBJECTS_H_


#include "texture.h"

class Ball;

class Generalobject
{
protected:
  int listno;
public:
  NxrVec3 pos;
  NxBody* nxbody;

  NxrReal dynFriction;
  NxrReal staFriction;

  bool isRacket;
  bool isBall;
  bool isTable;
  bool isGround;
  bool isNet;

  void updatePosition();
  void updateBody();

  void render();
  void renderInit();
};

class Movingobject : public Generalobject
{
public:
  NxrReal mass;
};


class Collisionobject : public Generalobject
{
public:
  virtual void checkCollision(Ball* bl) = 0;
};

class Boxobject : public Collisionobject
{
protected:
  NxrVec3 norm;
  NxrVec3 cont;
public:
  float dimensions[3];
  float m_orientationmatrix[16];

public:
  void checkCollision(Ball* bl);
};


class Table : public Boxobject
{
private:
  Texture *texUpper;
public:
  Table();
  ~Table();
  void render();
  void renderInit();

  void calculateOrientationMatrix();
};


class Table2 : public Boxobject
{
private:
  Texture *texUpper;
  float angle;
public:
  Table2();
  ~Table2();
  void render();
  void renderInit();
  NxBox* nxbox;

  void setUp();
  void setDown();

  void calculateOrientationMatrix();
  
};

class Net : public Boxobject
{
public:
	Net();
	void render();
	void calculateOrientationMatrix();
};

class Ball : public Movingobject
{
private:
  GLUquadricObj *b;
public:
  Ball();
  ~Ball();
  void render();
  void renderInit();
  void drawShadow();
  void updatePosition();

  float color[3];
  
  NxcReal r;
  NxLineSegment* nxline;

  NxrVec3 lastPos;
};

class Ground : public Collisionobject
{
private:
  Texture *texGround;
  Texture *texHorizon;
  Texture *texGreen;
public:
  Ground();
  ~Ground();
  void render();
  void checkCollision(Ball* bl);

  NxPlane* nxplane;
};

#endif