// pingpong.cpp : Defines the entry point for the application.
//



//////////////////////////////////////////////////////////////////
//                       Ping-Pong 3D Demo                      //
//                                                              //
//                                                              //
//////////////////////////////////////////////////////////////////
//                                                              //
// DESCRIPTION:                                                 //
//                                                              //
// OpenGL 3D Demo of a table-tennis simulation                  //
// Uses the NovodeX RigidBody SDK for the calculation of the    //
// Ball's physics and interaction with the other objects of     //
// the scene.                                                   //
//                                                              //
// AUTHORS:                                                     //
//                                                              //
// Christian Felix                                              //
// Martin Gernss                                                //
//                                                              //
//                                                              //
//                                                              //
// The demo was created during the physics-based simulation     //
// course at the Computer Graphics Laboratory of the Computer   //
// Science departement at ETH.                                  //
//                                                              //
// Special Thanks to NovodeX who provided their SDK             //
//                                                              //
// Zuerich, 23.7.2003                                           //
//                                                              //
//////////////////////////////////////////////////////////////////





#include "stdafx.h"
//#include "mmsystem.h"
#include "pingpong.h"
#include "objects.h"
#include "scene.h"
//#include "ego.h"

HGLRC hglRC;
BITMAP g_bm;

//NxFoundationSDK* fsdk;
//NxCollisionSDK* csdk;

//extern NxFoundationSDK* fsdk;
//extern NxCollisionSDK* csdk;

float g_angle = 0.0;


Scene sc;
float xpos,ypos;
bool keys[256];

App g_app;

GLfloat light_ambient[] = {0.1f,0.1f,0.1f,1};
GLfloat light_diffuse[] = {1.0, 1.0, 1.0, 0.4f};

GLfloat light_position[] = {50.0, 50.0, -50.0, 0.0};

void SetupGL(HWND hWnd) {
  PIXELFORMATDESCRIPTOR pfd = { 
  sizeof(PIXELFORMATDESCRIPTOR),   // size of this pfd 
  1,                     // version number 
  PFD_DRAW_TO_WINDOW |   // support window 
  PFD_SUPPORT_OPENGL |   // support OpenGL 
  PFD_DOUBLEBUFFER,      // double buffered 
  PFD_TYPE_RGBA,         // RGBA type 
  24,                    // 24-bit color depth 
  0, 0, 0, 0, 0, 0,      // color bits ignored 
  0,                     // no alpha buffer 
  0,                     // shift bit ignored 
  0,                     // no accumulation buffer 
  0, 0, 0, 0,            // accum bits ignored 
  32,                    // 32-bit z-buffer 
  0,                     // no stencil buffer 
  0,                     // no auxiliary buffer 
  PFD_MAIN_PLANE,        // main layer 
  0,                     // reserved 
  0, 0, 0                // layer masks ignored 
  };

  HDC hDC = GetDC(hWnd);
  int iPixelFormat = ChoosePixelFormat(hDC, &pfd);
  SetPixelFormat(hDC, iPixelFormat, &pfd);
}

void RenderGL(HWND hWnd) {
  PAINTSTRUCT ps;

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  glClearColor(0.5f,0.5f,1,1);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  gluLookAt(g_app.ego.eyeposition[0],g_app.ego.eyeposition[1],g_app.ego.eyeposition[2], 0,0,1, 0,1,0);
  
  glLightfv(GL_LIGHT0,GL_POSITION, light_position);

  glCallList(1000);
  sc.renderDynamic();

  glFlush();

  HDC hDC = BeginPaint(hWnd, &ps);
  wglSwapLayerBuffers(hDC,WGL_SWAP_MAIN_PLANE);
  EndPaint(hWnd, &ps);
}

void ResizeGL(int w, int h) {
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(50,(float)w/(float)h,0.1,10);
  g_app.mouseXFactor = w/4;
}

void InitGL() {
  glEnable(GL_NORMALIZE);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0,GL_AMBIENT,light_ambient);

  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR );	// Linear Filtering
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );	// Linear Filtering  
}

LRESULT CALLBACK WindowProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
  HDC hDC;
  PAINTSTRUCT ps;
  int height,width;

  switch(uMsg) {
  case WM_ACTIVATE:
    g_app.mcap.Start();
    break;
  case WM_CREATE:
    SetupGL(hWnd);
    hDC = BeginPaint(hWnd,&ps);
    hglRC = wglCreateContext (hDC);
    wglMakeCurrent (hDC, hglRC);
    EndPaint(hWnd,&ps);
  break;

  case WM_PAINT:
    RenderGL(hWnd);
    break;

  case WM_SIZE:
    height = HIWORD(lParam);
    width  = LOWORD(lParam);
    glViewport(0, 0, width, height);
    ResizeGL(width,height);
    InvalidateRect(hWnd,NULL,TRUE);
    break;
        
  case WM_TIMER:
    sc.doTimeStep();
  break;

  case WM_KEYUP:
			keys[wParam] = FALSE;
			break;

  case WM_KEYDOWN:
	  keys[wParam] = TRUE;
    if ( (int)wParam == 'W' ) {
      sc.reset();
    }
    else if ( (int)wParam == 'R' ) {
      sc.tableOrientation(true);
    }
    else if ( (int)wParam == 'F' ) {
      sc.tableOrientation(false);
    }
    else if ( (int)wParam == 'P' ) {
      if (g_app.mcap.IsCapturing()) {
        g_app.mcap.Stop();
        KillTimer(hWnd,ID_TIMER0);
      }
      else {
        g_app.mcap.Start();
        SetTimer(hWnd,ID_TIMER0,1,NULL);
      }
	  }
	    else if ( (int)wParam == 'Q' ) {
	    PostQuitMessage(0);
	  }
    break;
  case WM_DESTROY:
    wglDeleteContext(hglRC);
    PostQuitMessage(0);
    break;
  }
  return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
  MSG msg;
  WNDCLASS wc;
  HWND hWnd;

  /*DEVMODE dm; // Fullscreenmode
  memset(&dm,0,sizeof(dm));
  //EnumDisplaySettings(NULL,ENUM_CURRENT_SETTINGS,&dm);
  dm.dmPelsHeight = 800;
  dm.dmPelsWidth = 600;
  dm.dmBitsPerPel = 24;
  dm.dmFields = DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
  //ChangeDisplaySettings(&dm,CDS_FULLSCREEN);
  if (ChangeDisplaySettings(&dm,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL) {
    MessageBox(NULL,"Failed to go fullscreen","Error",MB_OK);
  }*/

  xpos = 0;
  g_app.mouseXFactor = 1000;//500;
 
  wc.style = CS_DBLCLKS | CS_OWNDC;
  wc.lpfnWndProc = WindowProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = NULL;
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = NULL;
  wc.lpszMenuName = NULL;
  wc.lpszClassName = "MartinGL1";

  if (!RegisterClass(&wc)) {
    return 0;
  }
  hWnd = CreateWindowEx(
      WS_EX_OVERLAPPEDWINDOW | WS_EX_APPWINDOW,
      "MartinGL1",
      "Ping-Pong 3D Demo",
      WS_POPUP | WS_OVERLAPPEDWINDOW | WS_VISIBLE,
      GetSystemMetrics(SM_CXSCREEN)/4,//0,
      GetSystemMetrics(SM_CYSCREEN)/4,//0,
      GetSystemMetrics(SM_CXSCREEN)/2,//800,//GetSystemMetrics(SM_CXSCREEN),
      GetSystemMetrics(SM_CYSCREEN)/2,//600,//GetSystemMetrics(SM_CYSCREEN),
      NULL,
      NULL,
      hInstance,
      NULL);

  g_app.hWnd = hWnd;
  g_app.hInstance = hInstance;

  g_app.mcap.Init(hInstance,hWnd);
  g_app.mcap.Start();


  InitGL();  // Initialize openGL

  sc.init();  // Initialize Scene

  glNewList(1000, GL_COMPILE);  // Prerender static scene to create display list
	sc.renderStatic();
	glEndList();

  SetTimer(hWnd,ID_TIMER0,1,NULL); // Setup timer

  while (true)
  {
    if (PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE)) {
      if( !GetMessage( &msg, NULL, 0, 0)) {
        return msg.wParam;
      }
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
    else {
	  if(keys[VK_RIGHT]) sc.rack->addSideAngle(-2.0f);  // Seperate Handle for keys
	  if(keys[VK_LEFT]) sc.rack->addSideAngle(2.0f);
	  if(keys[VK_UP]) sc.rack->increaseShootAngle(5.0f);
	  if(keys[VK_DOWN]) sc.rack->increaseShootAngle(-5.0f);
      if(keys['A']) {
		  g_app.ego.eyeposition[0] += 0.02f;
		  RenderGL(hWnd);
	  }
	  if(keys['S']) {
		  g_app.ego.eyeposition[0] -= 0.02f;
		  RenderGL(hWnd);
	  }
      if(keys['E']) {
		  g_app.ego.eyeposition[1] += 0.02f;
		  RenderGL(hWnd);
	  }
	  if(keys['D']) {
		  g_app.ego.eyeposition[1] -= 0.02f;
		  RenderGL(hWnd);
	  }
      InvalidateRect(hWnd,NULL,TRUE);
    }
  }

  sc.finish();  // Cleanup scene

  g_app.mcap.Stop();  // Stop mouse capturing

  wglMakeCurrent (NULL, NULL) ; // Cleanup openGL
  wglDeleteContext (hglRC); 
	return 0;
}

// Global helper to create 3D vector
void SetCoords(float f[], float f0, float f1, float f2) {
  f[0] = f0;
  f[1] = f1;
  f[2] = f2;
}
