// racket.cpp

#include "stdafx.h"

#include "pingpong.h"
#include "objects.h"
#include "racket.h"

Racket::Racket() {
  isRacket = true;
  isBall = false;
  isTable = false;
  isGround = false;
  isNet = false;
  dynFriction = 0.05f;
  staFriction = 0.05f;
  
  zyl = gluNewQuadric();
  pos.data.x = 0.0f;
  pos.data.y = 1.2f;
  pos.data.z = -0.8f;

  SetCoords(dimensions,0.06f,0.05f,0.01f);



  mass = 100.0f;

  nxbody = g_app.rsdk->createBody();
  nxbody->setPosition(pos);
  nxbody->setMass(mass);
  nxbody->setUserData((void*) this);


  shootangle = 0;
  sideangle = 0.0f;
  shootingspeed = 1;
  g_app.shootpower = 1;

  calculateOrientationMatrix();

  listno = glGenLists(1);
  glNewList(listno, GL_COMPILE);
	renderInit();
	glEndList();
}
Racket::~Racket() {
}

void Racket::render() {
  glPushMatrix();

  glMultMatrixf(m_orientationmatrix);

  glColor3f(g_app.shootpower/5,0,0);
  
  glCallList(listno);
  glPopMatrix();
}

void Racket::renderInit() {

  // Vorderseite
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(0,0,-1);
   //glColor3f(1,0,0);
   glVertex3f(-0.06f, 0.05f,-0.005f);
   glVertex3f( 0.06f, 0.05f,-0.005f);
   glVertex3f( 0.06f,-0.05f,-0.005f);
   glVertex3f(-0.06f,-0.05f,-0.005f);
  glEnd();
  // Rckseite
  glBegin(GL_TRIANGLE_FAN);
   glNormal3f(0,0,1);
   glColor3f(0,0,0);
   glVertex3f(-0.06f, 0.05f,0.005f);
   glVertex3f( 0.06f, 0.05f,0.005f);
   glVertex3f( 0.06f,-0.05f,0.005f);
   glVertex3f(-0.06f,-0.05f,0.005f);
  glEnd();

  // Kanten
  glColor3f(1,1,0);
  glBegin(GL_TRIANGLE_FAN);
    glNormal3f(0,1,0);
    glVertex3f(-0.06f, 0.05f, 0.005f);
    glVertex3f(-0.06f, 0.05f,-0.005f);
    glVertex3f( 0.06f, 0.05f,-0.005f);
    glVertex3f( 0.06f, 0.05f, 0.005f);
  glEnd();
  glBegin(GL_TRIANGLE_FAN);
    glNormal3f(1,0,0);
    glVertex3f( 0.06f, 0.05f,-0.005f);
    glVertex3f( 0.06f, 0.05f, 0.005f);
    glVertex3f( 0.06f,-0.05f, 0.005f);
    glVertex3f( 0.06f,-0.05f,-0.005f);
  glEnd();
  glBegin(GL_TRIANGLE_FAN);
    glNormal3f(0,-1,0);
    glVertex3f( 0.06f,-0.05f,-0.005f);
    glVertex3f( 0.06f,-0.05f, 0.005f);
    glVertex3f(-0.06f,-0.05f, 0.005f);
    glVertex3f(-0.06f,-0.05f,-0.005f);
  glEnd();
  glBegin(GL_TRIANGLE_FAN);
    glNormal3f(-1,0,0);
    glVertex3f(-0.06f, 0.05f, 0.005f);
    glVertex3f(-0.06f, 0.05f,-0.005f);
    glVertex3f(-0.06f,-0.05f,-0.005f);
    glVertex3f(-0.06f,-0.05f, 0.005f);
  glEnd();

  glColor3f(1,1,0.6f);

  glPushMatrix();

    glTranslatef(-0.12f,0,0);
    glRotatef(90,0,1,0);
    
    gluCylinder(zyl,0.01f,0.009f,0.08,8,4);
  glPopMatrix();  

}

void Racket::calculateOrientationMatrix() {
  float *m;
  restrictShootAngle();
  glPushMatrix();
   glLoadIdentity();
   glTranslatef(pos.data.x, pos.data.y, pos.data.z);
   glRotatef(shootangle/2,1,0,0);
   glRotatef(sideangle,0.0f,1.0f,0.0f);
   glGetFloatv(GL_MODELVIEW_MATRIX,m_orientationmatrix);
  glPopMatrix();
  m = this->m_orientationmatrix;

  NxrMat33 rotmat;
  rotmat(0,0) = m_orientationmatrix[0];
  rotmat(0,1) = m_orientationmatrix[4];
  rotmat(0,2) = m_orientationmatrix[8];

  rotmat(1,0) = m_orientationmatrix[1];
  rotmat(1,1) = m_orientationmatrix[5];
  rotmat(1,2) = m_orientationmatrix[9];

  rotmat(2,0) = m_orientationmatrix[2];
  rotmat(2,1) = m_orientationmatrix[6];
  rotmat(2,2) = m_orientationmatrix[10];
  nxbody->setOrientation(rotmat);
}

void Racket::setShootAngle(float a) {
  shootangle = a;
  calculateOrientationMatrix();
}

float Racket::getShootAngle() {
  return shootangle;
}

void Racket::increaseShootAngle(float da) {
  shootangle += da;
  calculateOrientationMatrix();
}

void Racket::restrictShootAngle() {
  if (shootangle < -70)
    shootangle = -70;
  else if (shootangle > 30)
    shootangle = 30;
}

void Racket::returnTo0ShootAngle() {
  if (shootangle<-4)
    shootangle += 4;
  else if (shootangle>4)
    shootangle -= 4;
  else
    shootangle = 0;
  calculateOrientationMatrix();
}

float Racket::getSideAngle()
{
	return sideangle;
}

void Racket::addSideAngle(float da)
{
	sideangle += da;
	if(sideangle > 45.0f) sideangle = 45.0f;
	if(sideangle < -45.0f) sideangle = -45.0f;
	calculateOrientationMatrix();
}


void Racket::restrictRange() {
  if (pos.data.x < - 1.5f) {
    pos.data.x = -1.5f;
  }
  else if (pos.data.x > 1.5f) {
    pos.data.x =  1.5f;
  }
  if (pos.data.z < - 3.5) {
    pos.data.z = -3.5;
  }
  else if (pos.data.z > -0.1) {
    pos.data.z =  -0.1f;
  }
}

void Racket::addPower(float dp) {
  if (g_app.shootpower < 8)
    g_app.shootpower += dp;
  else
    g_app.shootpower = 8;
}

void Racket::reducePower(float dp) {
  if (g_app.shootpower > 0)
    g_app.shootpower -= dp;
  else
    g_app.shootpower = 0;
}

void Racket::normPower() {
  if (g_app.shootpower > 2.2f)
    g_app.shootpower -= 0.2f;
  else
    g_app.shootpower = 2.0f;
}
